@@ -1,16 +1,27 @@
 package JavaScript::V8::Handlebars;
 
-use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
-use File::ShareDir qw/module_dir/;
 use File::Slurp qw/slurp/;
 use File::Spec;
+use File::Find qw/find/;
 use JavaScript::V8;
 
+use File::ShareDir qw/module_dir/;
+	my $module_dir = module_dir( __PACKAGE__ );
+	my $JS_FILE = glob "$module_dir/*.js"; #This has global state, by the way
+
+
+###### Dynamic methods #####################
+	for my $meth ( qw/safeString escapeString registerPartial/ ) {
+		no strict 'refs';
+			*$meth = sub { $_[0]->{$meth}->(@_[1..$#_]) };
+	}
+##############################################
+
 sub new {
 	my( $class, @opts ) = @_;
 
@@ -26,24 +37,18 @@ sub _build_context {
 
 	my $c = $self->{c} = JavaScript::V8::Context->new;
 
-	my $module_dir = module_dir( __PACKAGE__ );
-	my $js_file = glob "$module_dir/*.js";
 
 	# slurp returns a list in list context..
-	$c->eval( scalar slurp $js_file );
+	$c->eval( scalar slurp $JS_FILE );
 	die $@ if $@;
 
 
+	# Store subrefs for each javascript method
 	for my $meth (qw/precompile registerHelper registerPartial template compile safeString escapeString/ ) {
 		$self->{$meth} = $c->eval( "Handlebars.$meth" );
 		die $@ if $@;
 	}
 
-	for my $meth ( qw/safeString escapeString registerPartial/ ) {
-		my $code = $self->{$meth};
-		no strict 'refs';
-			*$meth = sub { shift; $code->(@_); };
-	}
 }
 
 sub c {
@@ -91,6 +96,7 @@ sub registerHelper {
 
 		$code =~ s/function/function $bind_name/;
 
+		#TODO Why do we name the function?
 		$self->eval($code);
 		$self->eval( "Handlebars.registerHelper('$name',$bind_name)" );
 	}
@@ -141,16 +147,33 @@ sub add_template {
 }
 
 sub add_template_file {
-	my( $self, $file ) = @_;
+	my( $self, $file, $base ) = @_;
+	if( $base ) { $file = File::Spec->rel2abs( $file, $base ); }
 
-	die "Failed to find $file $!" unless -e $file and -r $file;
+	die "Failed to read $file $!" unless -e $file and -r $file;
 	
-	my $name = (File::Spec->splitdir($file))[-1];
+	my $name = File::Spec->abs2rel( $file, $base );
 		$name =~ s/\..*$//;
-	
+
 	$self->add_template( $name, scalar slurp $file );
 }
 
+sub add_template_dir {
+	my( $self, $dir, $ext ) = @_;
+	$ext ||= 'hbs';
+
+	find( { wanted => sub {
+			return unless -f;
+			return unless /\.$ext$/;
+			warn "Can't read $_" unless -r;
+
+			$self->add_template_file( File::Spec->rel2abs($_), $dir );
+		},
+		no_chdir => 1,
+	}, $dir );
+
+}
+
 sub execute_template {
 	my( $self, $name, $args ) = @_;
 
@@ -175,11 +198,11 @@ sub precompiled {
 
 =head1 NAME
 
-JavaScript::V8::Handlebars - The great new JavaScript::V8::Handlebars!
+JavaScript::V8::Handlebars - Compile and execute Handlebars templates via the actual JS library
 
 =head1 VERSION
 
-Version 0.01
+Version 0.03
 
 =head1 AUTHOR
 
@@ -201,30 +224,6 @@ You can find documentation for this module with the perldoc command.
 perldoc JavaScript::V8::Handlebars
 
 
-You can also look for information at:
-
-=over 4
-
-=item * RT: CPAN's request tracker (report bugs here)
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=JavaScript-V8-Handlebars>
-
-=item * AnnoCPAN: Annotated CPAN documentation
-
-L<http://annocpan.org/dist/JavaScript-V8-Handlebars>
-
-=item * CPAN Ratings
-
-L<http://cpanratings.perl.org/d/JavaScript-V8-Handlebars>
-
-=item * Search CPAN
-
-L<http://search.cpan.org/dist/JavaScript-V8-Handlebars/>
-
-=back
-
-
-
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2015 Robert Grimes.
@@ -266,4 +265,8 @@ CONSEQUENTIAL DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THE PACKAGE,
 EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
+=head1 SEE ALSO
+
+L<https://github.com/rmzg/JavaScript-V8-Handlebars>
+
 =cut
